test_log_store = '/Users/leqee/code/nehushtan/log'

MYSQL_CONFIG = {
    'host': '172.16.1.52',
    'port': 3306,
    'user': 'ljni',
    'password': 'riIOTtIdBbT1',
    'db': 'sinri',
    'charset': 'utf8',
    'auto_commit': True,
}

IMAP_CONFIG = {
    # ...
}

SMTP_CONFIG = {
    # ...
}
