#  Copyright (c) 2020. Sinri Edogawa

MYSQL_QUERY_STATUS_INIT = "INIT"
MYSQL_QUERY_STATUS_QUERIED = "QUERIED"
MYSQL_QUERY_STATUS_EXECUTED = "EXECUTED"
MYSQL_QUERY_STATUS_STREAMING = "STREAMING"
MYSQL_QUERY_STATUS_STREAMED = "STREAMED"
MYSQL_QUERY_STATUS_ERROR = "ERROR"

MYSQL_QUERY_ROW_TYPE_DICT = 'DICT'
MYSQL_QUERY_ROW_TYPE_TUPLE = 'TUPLE'
MYSQL_QUERY_ROW_TYPE_UNKNOWN = 'UNKNOWN'
